package org.richfaces.renderkit.html.images;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.util.Date;
import javax.faces.context.FacesContext;
import org.ajax4jsf.resource.InternetResourceBuilder;
import org.ajax4jsf.resource.Java2Dresource;
import org.ajax4jsf.resource.PngRenderer;
import org.ajax4jsf.resource.ResourceContext;
import org.ajax4jsf.util.HtmlColor;
import org.ajax4jsf.util.Zipper2;
import org.richfaces.skin.Skin;
import org.richfaces.skin.SkinFactory;

public class GrowlBackground extends Java2Dresource {

    private static Dimension dimension = new Dimension(602, 310);

    public GrowlBackground() {
        setRenderer(new PngRenderer());
        setLastModified(new Date(InternetResourceBuilder.getInstance().getStartTime()));
    }

    @Override
    public Dimension getDimensions(FacesContext facesContext, Object data) {
        return dimension;
    }

    @Override
    protected Dimension getDimensions(ResourceContext resourceContext) {
        return dimension;
    }

    @Override
    protected Object getDataToStore(FacesContext context, Object data) {
        Skin skin = SkinFactory.getInstance().getSkin(context);
        Skin defaultSkin = SkinFactory.getInstance().getDefaultSkin(context);

        String skinParameter = "headerBackgroundColor";
        String backgroundColor = (String) skin.getParameter(context, skinParameter);
        if (null == backgroundColor || "".equals(backgroundColor)) {
            backgroundColor = (String) defaultSkin.getParameter(context, skinParameter);
        }
        if (backgroundColor == null) {
            return null;
        }
        skinParameter = "panelBorderColor";
        String borderColor = (String) skin.getParameter(context, skinParameter);
        if (null == borderColor || "".equals(borderColor)) {
            borderColor = (String) defaultSkin.getParameter(context, skinParameter);
        }
        if (borderColor == null) {
            return null;
        }
        byte[] ret = new byte[6];
        Zipper2 zipper2 = new Zipper2(ret).addColor(HtmlColor.decode(backgroundColor).getRGB());
        zipper2.addColor(HtmlColor.decode(borderColor).getRGB());

        return ret;
    }

    @Override
    protected Object deserializeData(byte[] objectArray) {
        if (objectArray == null) {
            return null;
        }
        Object[] colors = new Object[2];
        Zipper2 z = new Zipper2(objectArray);
        colors[0] = z.nextColor();
        colors[1] = z.nextColor();

        return colors;
    }

    @Override
    public void paint(ResourceContext context, Graphics2D graphics) {
        Object[] stored = (Object[]) restoreData(context);
        if (stored != null) {
            Dimension dim = getDimensions(context);
            BufferedImage image = paintImage(stored);
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
            graphics.drawImage(image, 0, 0, dim.width, dim.height, null);
        }
    }

    public BufferedImage paintImage(Object[] colors) {
        int width = dimension.width;
        int height = dimension.height;
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2d = image.createGraphics();
        Color activeColor = (Color) colors[0];
        Color borderColor = (Color) colors[1];
        activeColor = new Color(activeColor.getRed(),activeColor.getGreen(),activeColor.getBlue(),170);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        g2d.setStroke(new BasicStroke(3));
        /**
         * Draw background rectangle without border
         */
        Shape shape = new RoundRectangle2D.Double(0, 30, 300, 280, 20, 20);
        g2d.setColor(activeColor);
        g2d.fill(shape);
        /**
         * Draw background rectangle with border
         */
        shape = new RoundRectangle2D.Double(301, 30, 300, 280, 20, 20);
        activeColor = new Color(activeColor.getRed(),activeColor.getGreen(),activeColor.getBlue(),255);
        g2d.setColor(activeColor);
        g2d.fill(shape);
        g2d.setColor(borderColor);
        g2d.draw(shape);
        /**
         * Draw shade under close button
         */
        int left = 2, top = 2;
        width = 26;
        height = 28;
        shape = new Ellipse2D.Double(left, top, width, height);
        Color gradientColor1 = new Color(activeColor.getRed(), activeColor.getGreen(), activeColor.getBlue(), 50);
        Color gradientColor2 = new Color(activeColor.getRed(), activeColor.getGreen(), activeColor.getBlue(), 30);
        g2d.setPaint(new RoundGradientPaint(left + width / 2, top + height / 2, gradientColor1, new Point2D.Double(8, 8), gradientColor2));
        g2d.fill(shape);
        /**
         * Draw close button
         */
        shape = new Ellipse2D.Double(4, 4, 20, 20);
        g2d.setColor(activeColor);
        g2d.fill(shape);
        g2d.setColor(borderColor);
        g2d.draw(shape);
        /**
         * Draw X in the middle of close button
         */
        int a = 12, b = 16;
        g2d.setStroke(new BasicStroke(2));
        g2d.drawLine(a, a, b, b);
        g2d.drawLine(a, b, b, a);

        g2d.dispose();
        return image;
    }

    /**
     * Some code from internet for making round gradient paint.
     */
    public static class RoundGradientPaint implements Paint {

        protected Point2D point;
        protected Point2D mRadius;
        protected Color mPointColor, mBackgroundColor;

        public RoundGradientPaint(double x, double y, Color pointColor,
                Point2D radius, Color backgroundColor) {
            if (radius.distance(0, 0) <= 0) {
                throw new IllegalArgumentException("Radius must be greater than 0.");
            }
            point = new Point2D.Double(x, y);
            mPointColor = pointColor;
            mRadius = radius;
            mBackgroundColor = backgroundColor;
        }

        public PaintContext createContext(ColorModel cm, Rectangle deviceBounds,
                Rectangle2D userBounds, AffineTransform xform, RenderingHints hints) {
            Point2D transformedPoint = xform.transform(point, null);
            Point2D transformedRadius = xform.deltaTransform(mRadius, null);
            return new RoundGradientContext(transformedPoint, mPointColor,
                    transformedRadius, mBackgroundColor);
        }

        public int getTransparency() {
            int a1 = mPointColor.getAlpha();
            int a2 = mBackgroundColor.getAlpha();
            return (((a1 & a2) == 0xff) ? OPAQUE : TRANSLUCENT);
        }
    }

    public static class RoundGradientContext implements PaintContext {

        protected Point2D mPoint;
        protected Point2D mRadius;
        protected Color color1, color2;

        public RoundGradientContext(Point2D p, Color c1, Point2D r, Color c2) {
            mPoint = p;
            color1 = c1;
            mRadius = r;
            color2 = c2;
        }

        public void dispose() {
        }

        public ColorModel getColorModel() {
            return ColorModel.getRGBdefault();
        }

        public Raster getRaster(int x, int y, int w, int h) {
            WritableRaster raster = getColorModel().createCompatibleWritableRaster(
                    w, h);

            int[] data = new int[w * h * 4];
            for (int j = 0; j < h; j++) {
                for (int i = 0; i < w; i++) {
                    double distance = mPoint.distance(x + i, y + j);
                    double radius = mRadius.distance(0, 0);
                    double ratio = distance / radius;
                    if (ratio > 1.0) {
                        ratio = 1.0;
                    }

                    int base = (j * w + i) * 4;
                    data[base + 0] = (int) (color1.getRed() + ratio * (color2.getRed() - color1.getRed()));
                    data[base + 1] = (int) (color1.getGreen() + ratio * (color2.getGreen() - color1.getGreen()));
                    data[base + 2] = (int) (color1.getBlue() + ratio * (color2.getBlue() - color1.getBlue()));
                    data[base + 3] = (int) (color1.getAlpha() + ratio * (color2.getAlpha() - color1.getAlpha()));
                }
            }
            raster.setPixels(0, 0, w, h, data);

            return raster;
        }
    }
}
