package org.richfaces.renderkit.html.images;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.util.Date;
import javax.faces.context.FacesContext;
import org.ajax4jsf.resource.InternetResourceBuilder;
import org.ajax4jsf.resource.Java2Dresource;
import org.ajax4jsf.resource.PngRenderer;
import org.ajax4jsf.resource.ResourceContext;
import org.ajax4jsf.util.HtmlColor;
import org.ajax4jsf.util.Zipper2;
import org.richfaces.renderkit.html.images.GrowlBackground.RoundGradientPaint;
import org.richfaces.skin.Skin;
import org.richfaces.skin.SkinFactory;

public class GrowlCloseButton extends Java2Dresource {

    private static Dimension dimension = new Dimension(22, 22);

    public GrowlCloseButton() {
        setRenderer(new PngRenderer());
        setLastModified(new Date(InternetResourceBuilder.getInstance().getStartTime()));
    }

    @Override
    public Dimension getDimensions(FacesContext facesContext, Object data) {
        return dimension;
    }

    @Override
    protected Dimension getDimensions(ResourceContext resourceContext) {
        return dimension;
    }

    @Override
    protected Object getDataToStore(FacesContext context, Object data) {
        Skin skin = SkinFactory.getInstance().getSkin(context);
        Skin defaultSkin = SkinFactory.getInstance().getDefaultSkin(context);

        String skinParameter = "headerBackgroundColor";
        String backgroundColor = (String) skin.getParameter(context, skinParameter);
        if (null == backgroundColor || "".equals(backgroundColor)) {
            backgroundColor = (String) defaultSkin.getParameter(context, skinParameter);
        }
        if (backgroundColor == null) {
            return null;
        }
        skinParameter = "panelBorderColor";
        String borderColor = (String) skin.getParameter(context, skinParameter);
        if (null == borderColor || "".equals(borderColor)) {
            borderColor = (String) defaultSkin.getParameter(context, skinParameter);
        }
        if (borderColor == null) {
            return null;
        }
        byte[] ret = new byte[6];
        Zipper2 zipper2 = new Zipper2(ret).addColor(HtmlColor.decode(backgroundColor).getRGB());
        zipper2.addColor(HtmlColor.decode(borderColor).getRGB());

        return ret;
    }

    @Override
    protected Object deserializeData(byte[] objectArray) {
        if (objectArray == null) {
            return null;
        }
        Object[] colors = new Object[2];
        Zipper2 z = new Zipper2(objectArray);
        colors[0] = z.nextColor();
        colors[1] = z.nextColor();

        return colors;
    }

    @Override
    public void paint(ResourceContext context, Graphics2D graphics) {
        Object[] stored = (Object[]) restoreData(context);
        if (stored != null) {
            Dimension dim = getDimensions(context);
            BufferedImage image = paintImage(stored);
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
            graphics.drawImage(image, 0, 0, dim.width, dim.height, null);
        }
    }

    public BufferedImage paintImage(Object[] colors) {
        int width = dimension.width;
        int height = dimension.height;
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2d = image.createGraphics();
        Color activeColor = (Color) colors[0];
        Color borderColor = (Color) colors[1];
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        g2d.setStroke(new BasicStroke(3));
        /**
         * Draw shade under close button
         */
        int left = 0, top = 0;
        width = 26;
        height = 28;
        Shape shape = new Ellipse2D.Double(left, top, width, height);
        Color gradientColor1 = new Color(activeColor.getRed(), activeColor.getGreen(), activeColor.getBlue(), 50);
        Color gradientColor2 = new Color(activeColor.getRed(), activeColor.getGreen(), activeColor.getBlue(), 30);
        g2d.setPaint(new RoundGradientPaint(left + width / 2, top + height / 2, gradientColor1, new Point2D.Double(8, 8), gradientColor2));
        g2d.fill(shape);
        /**
         * Draw close button
         */
        shape = new Ellipse2D.Double(2, 2, 20, 20);
        g2d.setColor(activeColor);
        g2d.fill(shape);
        g2d.setColor(borderColor);
        g2d.draw(shape);
        /**
         * Draw X in the middle of close button
         */
        int a = 10, b = 14;
        g2d.setStroke(new BasicStroke(2));
        g2d.drawLine(a, a, b, b);
        g2d.drawLine(a, b, b, a);

        g2d.dispose();
        return image;
    }

}
